/*
 * Copy me if you can.
 * by 20h
 */

#include <u.h>
#include <libc.h>
#include "xmlpull.h"
#include "ysearch.h"

char *
readline(int s)
{
	char *ret;
	int l;

	ret = malloc(0);
	l = 0;

	while((ret = realloc(ret, ++l)) != nil && read(s, &ret[l - 1], 1) > 0 &&
			l < 512){
		if(l > 1){
			if(ret[l - 2] == '\r'){
				ret[l - 1] = '\0';
				ret[l - 2] = '\0';
				return ret;
			}
		}
	}

	if(ret != nil){
		ret = realloc(ret, ++l);
		ret[l - 1] = '\0';
		return ret;
	}

	return nil;
}

char *
readhttphdr(int s)
{
	char *ret;

	while((ret = readline(s)) != nil && ret[0] != '\0')
		free(ret);

	if(ret == nil)
		return nil;

	ret = readline(s);
	if(ret == nil)
		return nil;
	free(ret);

	ret = readline(s);

	return ret;
}

void
usage(void)
{
	print("usage:\n"
		  "common: [-x net] [-a appid] [-s search] [-r start] [-n results] searchstring\n"
		  "search:   web: [-t type] [-f format] [-d adult_ok] [-i similar_ok] [-l lang]\n"
		  "        video: [-t type] [-f format] [-d adult_ok]\n"
		  "         news: [-t type] [-o sort] [-l lang]\n"
		  "        local: [-u radius] [-e street] [-y city] [-w state] [-z zip] [-c location]\n"
		  "        image: [-t type] [-f format] [-d adult_ok]\n"
		  "\nPowered by Yahoo! Search. (As the Terms of Use wants it.)\n");
	exits(0);
}

void
parseresponse(int sock)
{
	xmlpull *x, *a;
	char st;

	st = NONE;

	x = openxmlpull(sock);
	if(x == nil)
		return;

	while((a = nextxmlpull(x)) != nil && st != END){
		switch(a->ev){
		case START_DOCUMENT:
			st = NONE;
			break;
		case START_TAG:
			if(!strcmp(x->na, "Result")){
				st = RESULT;
				break;
			}
			if(!strcmp(x->na, "Title") && st == RESULT){
				st = TITLE;
				break;
			}
			if(!strcmp(x->na, "Summary") && st == RESULT){
				st = SUMMARY;
				break;
			}
			if(!strcmp(x->na, "Url") && st == RESULT){
				st = URL;
				break;
			}
			if(!strcmp(x->na, "MimeType") && st == RESULT){
				st = MIME;
				break;
			}
			if(!strcmp(x->na, "Cache") && st == RESULT){
				st = CACHE;
				break;
			}
			if(!strcmp(x->na, "Error")){
				st = ERROR;
				break;
			}
			if(!strcmp(x->na, "Message")){
				st = ERROR_MSG;
				break;
			}
			break;
		case START_END_TAG:
			break;
		case TEXT:
			switch(st){
			case TITLE:
				print("Title: %s\n", x->na);
				break;
			case SUMMARY:
				print("Summary: \n%s\n", x->na);
				break;
			case URL:
				print("Url: %s\n", x->na);
				break;
			case MIME:
				print("Mime: %s\n", x->na);
				break;
			case ERROR_MSG:
				print("Error: %s\n", x->na);
			default:
				break;
			}
			break;
		case ATTR:
			break;
		case END_TAG:
			if(!strcmp(x->na, "Result") && st == RESULT){
				st = NONE;
				print("---\n");
				break;
			}
			if(!strcmp(x->na, "Title") && st == TITLE){
				st = RESULT;
				break;
			}
			if(!strcmp(x->na, "Summary") && st == SUMMARY){
				st = RESULT;
				break;
			}
			if(!strcmp(x->na, "Url") && st == URL){
				st = RESULT;
				break;
			}
			if(!strcmp(x->na, "MimeType") && st == MIME){
				st = RESULT;
				break;
			}
			if(!strcmp(x->na, "Cache") && st == CACHE){
				st = RESULT;
				break;
			}
			if(!strcmp(x->na, "Error") && st == ERROR){
				st = NONE;
				break;
			}
			if(!strcmp(x->na, "Message") && st == ERROR_MSG){
				st = NONE;
				break;
			}
			break;
		case END_DOCUMENT:
			st = END;
			break;
		default:
			print("Please inform the xmlpull author about this: %x\n", x->ev);
			break;
		}
	}
	freexmlpull(x);

	return;
}

void
main(int argc, char *argv[])
{
	char *net, *appid, *search, *start, *type, *format, *adultok, *similarok, *lang, *sort,
			*radius, *street, *city, *state, *zip, *location, *num, *query;
	int sock;

	net = "/net/tcp";
	appid = "YahooDemo";
	search = "web";
	start = "1";
	type = "all";
	format = "all";
	adultok = "1";
	similarok = "1";
	lang = "de";
	sort = "date";
	radius = "";
	street = "";
	city = "";
	state = "";
	zip = "";
	location = "";
	num = "50";

	ARGBEGIN {
		case 'l':
			lang = EARGF(usage());
			break;
		case 'a':
			appid = EARGF(usage());
			break;
		case 't':
			type = EARGF(usage());
			break;
		case 's':
			search = EARGF(usage());
			break;
		case 'r':
			start = EARGF(usage());
			break;
		case 'n':
			num = EARGF(usage());
			break;
		case 'f':
			format = EARGF(usage());
			break;
		case 'd':
			adultok = EARGF(usage());
			break;
		case 'i':
			similarok = EARGF(usage());
			break;
		case 'o':
			sort = EARGF(usage());
			break;
		case 'u':
			radius = EARGF(usage());
			break;
		case 'e':
			street = EARGF(usage());
			break;
		case 'y':
			city = EARGF(usage());
			break;
		case 'w':
			state = EARGF(usage());
			break;
		case 'z':
			zip = EARGF(usage());
			break;
		case 'c':
			location = EARGF(usage());
			break;
		case 'x':
			net = EARGF(usage());
			break;
		default:
			usage();
	} ARGEND;

	if(argc == 0)
		usage();
	query = argv[0];

	if(!strcmp(appid, "YahooDemo"))
		print("You are using YahooDemo for this, but I certainly do not know"
			  " whether there is an limit to this account, so be careful.\n");

	sock = dial(netmkaddr("api.search.yahoo.com", net, "80"), 0, 0, 0);
	if(sock < 0)
		sysfatal("dial: %r");

	if(!strcmp(search, "web"))
		fprint(sock, "GET /WebSearchService/V1/webSearch?appid=%s&query=%s&type=%s&results=%s"
					 "&start=%s&format=%s&adult_ok=%s&similar_ok=%s&language=%s HTTP/1.1\r\n",
					 appid, query, type, num, start, format, adultok, similarok, lang);
	if(!strcmp(search, "video"))
		fprint(sock, "GET /VideoSearchService/V1/videoSearch?appid=%s&query=%s&type=%s"
				     "&results=%s&start=%s&format=%s&adult_ok=%s HTTP/1.1\r\n",
					 appid, query, type, num, start, format, adultok);
	if(!strcmp(search, "news"))
		fprint(sock, "GET /NewsSearchService/V1/newsSearch?appid=%s&query=%s&type=%s"
					 "&results=%s&start=%s&sort=%s&language=%s HTTP/1.1\r\n",
					 appid, query, type, num, start, sort, lang);
	if(!strcmp(search, "local"))
		fprint(sock, "GET /LocalSearchService/V1/localSearch?appid=%s&query=%s&type=%s"
					 "&results=%s&start=%s&radius=%s&street=%s&city=%s&state=%s&zip=%s"
					 "&location=%s HTTP/1.1\r\n", appid, query, type, num, start,
					 radius, street, city, state, zip, location);
	if(!strcmp(search, "image"))
		fprint(sock, "GET /ImageSearchService/V1/imageSearch?appid=%s&query=%s&type=%s"
					 "&results=%s&start=%s&format=%s&adult_ok=%s HTTP/1.1\r\n",
					 appid, query, type, num, start, format, adultok);

	fprint(sock, "Host: api.search.yahoo.com\r\nUser-Agent: ysearch (Plan9)\r\n\r\n");

	net = readhttphdr(sock);
	if(net == nil)
		sysfatal("readhttpdr: %r");
	free(net);

	parseresponse(sock);

	close(sock);
	exits(0);
}
